<!doctype html>
<html lang="en">
  <head>
    <title>Home</title>
    <style>{css}</style>
  </head>
  <body>
    <div class="header">
      <div class="menu">
        {menu}
      </div>
      <span id="clock">00:00:00</span>
    </div>
    {html}
  </body>

  <script>
    function time() {
      var d = new Date();
      var s = d.getSeconds();
      var m = d.getMinutes();
      var h = d.getHours();

      document.getElementById("clock").innerHTML =
        ("0" + h).substr(-2) +
        ":" +
        ("0" + m).substr(-2) +
        ":" +
        ("0" + s).substr(-2);
    }

    setInterval(time, 1000);
    document.getElementById("q").focus();
  </script>
</html>
